@@ -21,12 +21,8 @@
 # Standard library imports
 import datetime
 import time         # 00000 - stopwatch elements
+import math
 from os import path
-from math import degrees as degrees
-from math import pi as pi
-from math import atan as atan
-from math import tan as tan
-from math import cos as cos
 
 # Third party imports
 from skyfield import VERSION
@@ -40,6 +36,7 @@ from skyfield.data import hipparcos
 import numpy as np
 
 # Local application imports
+#import config
 from sfalmanac.lib import config as config
 
 #----------------------
@@ -300,7 +297,7 @@ def sunSD(d):               # used in sunmoontab(m)
     position = earth.at(t12).observe(sun)
     distance = position.apparent().radec(epoch='date')[2]
     dist_km = distance.km
-    sds = degrees(atan(695500.0 / dist_km))   # radius of sun = 695500 km
+    sds = math.degrees(math.atan(695500.0 / dist_km))   # radius of sun = 695500 km
     sdsm = "{:0.1f}".format(sds * 60)   # convert to minutes of arc
 
     t0 = ts.ut1(d.year, d.month, d.day, 0, 0, 0)
@@ -319,7 +316,7 @@ def moonSD(d):              # used in sunmoontab(m)
     position = earth.at(t12).observe(moon)
     distance = position.apparent().radec(epoch='date')[2]
     dist_km = distance.km
-    sdm = degrees(atan(1738.1/dist_km))   # equatorial radius of moon = 1738.1 km
+    sdm = math.degrees(math.atan(1738.1/dist_km))   # equatorial radius of moon = 1738.1 km
     sdmm = "{:0.1f}".format(sdm * 60)  # convert to minutes of arc
     return sdmm
 
@@ -363,7 +360,7 @@ def moonGHA(d, round2seconds = False):  # used in sunmoontab(m) & equationtab (i
         decm[i] = fmtdeg(dec.degrees[i],2)
         degm[i] = dec.degrees[i]
         dist_km = distance.km[i]
-        HP = degrees(atan(6378.0/dist_km))	    # radius of earth = 6378.0 km
+        HP = math.degrees(math.atan(6378.0/dist_km))	# radius of earth = 6378.0 km
         HPm[i] = "{:0.1f}'".format(HP * 60)     # convert to minutes of arc
 
     # degm has been added for the sunmoontab function
@@ -372,7 +369,7 @@ def moonGHA(d, round2seconds = False):  # used in sunmoontab(m) & equationtab (i
 
     return gham, decm, degm, HPm, GHAupper, GHAlower, ghaSoD, ghaEoD
 
-def moonVD(d0,d):           # used in sunmoontab(m)
+def moonVD(d0, d):           # used in sunmoontab(m)
     # first value required is from 23:30 on the previous day...
     t0 = ts.ut1(d0.year, d0.month, d0.day, 23, 30, 0)
     pos0 = earth.at(t0).observe(moon)
@@ -597,14 +594,19 @@ def planetstransit(d, round2seconds = False):      # used in starstab & meridian
     ra0 = position0.apparent().radec(epoch='date')[0]	# RA
     vau = position0.apparent().radec(epoch='date')[2]	# distance
     vsha = fmtgha(0, ra0.hours)
-    hpvenus = "{:0.1f}".format((tan(6371/(vau.au*149597870.7)))*60*180/pi)
+    hpvenus = "{:0.1f}".format((math.tan(6371/(vau.au*149597870.7)))*60*180/math.pi)
 
     # calculate planet transit
     tfr = t0
     tto = ts.ut1(d1.year, d1.month, d1.day, 0, 0, 0)
     position = earth.at(tfr).observe(venus)
     ra = position.apparent().radec(epoch='date')[0]
+    #print('Venus transit: ', tfr.gast, ra.hours)
+    start00 = time.time()                   # 00000
     transit_time, y = almanac.find_discrete(tfr, tto, planet_transit(venus))
+    config.stopwatch += time.time()-start00 # 00000
+    #if len(transit_time) != 1:
+    #    print('returned %s values' %len(transit_time))
     vtrans = rise_set(transit_time,y,u'Venus   0{} E transit'.format(degree_sign),round2seconds)[0]
 
 # Mars
@@ -612,14 +614,16 @@ def planetstransit(d, round2seconds = False):      # used in starstab & meridian
     ra0 = position0.apparent().radec(epoch='date')[0]	# RA
     mau = position0.apparent().radec(epoch='date')[2]	# distance
     marssha = fmtgha(0, ra0.hours)
-    hpmars = "{:0.1f}".format((tan(6371/(mau.au*149597870.7)))*60*180/pi)
+    hpmars = "{:0.1f}".format((math.tan(6371/(mau.au*149597870.7)))*60*180/math.pi)
 
     # calculate planet transit
     tfr = t0
     tto = ts.ut1(d1.year, d1.month, d1.day, 0, 0, 0)
     position = earth.at(tfr).observe(mars)
     ra = position.apparent().radec(epoch='date')[0]
+    start00 = time.time()                   # 00000
     transit_time, y = almanac.find_discrete(tfr, tto, planet_transit(mars))
+    config.stopwatch += time.time()-start00 # 00000
     marstrans = rise_set(transit_time,y,u'Mars    0{} E transit'.format(degree_sign),round2seconds)[0]
 
 # Jupiter
@@ -632,7 +636,9 @@ def planetstransit(d, round2seconds = False):      # used in starstab & meridian
     tto = ts.ut1(d1.year, d1.month, d1.day, 0, 0, 0)
     position = earth.at(tfr).observe(jupiter)
     ra = position.apparent().radec(epoch='date')[0]
+    start00 = time.time()                   # 00000
     transit_time, y = almanac.find_discrete(tfr, tto, planet_transit(jupiter))
+    config.stopwatch += time.time()-start00 # 00000
     jtrans = rise_set(transit_time,y,u'Jupiter 0{} E transit'.format(degree_sign),round2seconds)[0]
     
 # Saturn
@@ -645,7 +651,9 @@ def planetstransit(d, round2seconds = False):      # used in starstab & meridian
     tto = ts.ut1(d1.year, d1.month, d1.day, 0, 0, 0)
     position = earth.at(tfr).observe(saturn)
     ra = position.apparent().radec(epoch='date')[0]
+    start00 = time.time()                   # 00000
     transit_time, y = almanac.find_discrete(tfr, tto, planet_transit(saturn))
+    config.stopwatch += time.time()-start00 # 00000
     sattrans = rise_set(transit_time,y,u'Saturn  0{} E transit'.format(degree_sign),round2seconds)[0]
     
     return [vsha,vtrans,marssha,marstrans,jsha,jtrans,satsha,sattrans,hpmars,hpvenus]
@@ -782,8 +790,7 @@ def twilight(d, lat, hemisph, round2seconds = False):  # used in twilighttab (se
     # Sunrise/Sunset...
     start00 = time.time()                   # 00000
     actual, y = almanac.find_discrete(t0, t1, daylength(locn, 0.8333))
-    stop00 = time.time()                    # 00000
-    config.stopwatch += stop00-start00      # 00000
+    config.stopwatch += time.time()-start00 # 00000
     out[2], out[3], r2, s2, fs = rise_set(actual,y,lats,round2seconds)
     if out[2] == '--:--' and out[3] == '--:--':	# if neither sunrise nor sunset...
         abhd = True                             # enable above/below horizon display
@@ -794,8 +801,7 @@ def twilight(d, lat, hemisph, round2seconds = False):  # used in twilighttab (se
     # Civil Twilight...
     start00 = time.time()                   # 00000
     civil, y = almanac.find_discrete(t0, t1, daylength(locn, 6.0))
-    stop00 = time.time()                    # 00000
-    config.stopwatch += stop00-start00      # 00000
+    config.stopwatch += time.time()-start00 # 00000
     out[1], out[4], r2, s2, fs = rise_set(civil,y,lats,round2seconds)
     if abhd and out[1] == '--:--' and out[4] == '--:--':	# if neither begin nor end...
         yn = midnightsun(d, hemisph)
@@ -805,8 +811,7 @@ def twilight(d, lat, hemisph, round2seconds = False):  # used in twilighttab (se
     # Nautical Twilight...
     start00 = time.time()                   # 00000
     naut, y = almanac.find_discrete(t0, t1, daylength(locn, 12.0))
-    stop00 = time.time()                    # 00000
-    config.stopwatch += stop00-start00      # 00000
+    config.stopwatch += time.time()-start00 # 00000
     out[0], out[5], r2, s2, fs = rise_set(naut,y,lats,round2seconds)
     if abhd and out[0] == '--:--' and out[5] == '--:--':	# if neither begin nor end...
         yn = midnightsun(d, hemisph)
@@ -851,7 +856,7 @@ def daylength(topos, degBelowHorizon):
 #global sunvisible
 #sunvisible = [[None]*3 for i in range(31)]	# sunvisible[0][0] up to sunvisible[30][2]
 
-##UNUSED - replaced by midnightsun ##
+##UNUSED - only called by getsunstate (obsolete) ##
 def sunstate(ndx, h):
     # return the current sunstate (if known)
     out = '--:--'
@@ -861,7 +866,7 @@ def sunstate(ndx, h):
         out = r'''\rule{12Pt}{4Pt}'''
     return out
 
-##UNUSED - poor algorithm ##
+##UNUSED - poor algorithm - replaced by midnightsun ##
 def getsunstate(dt, lat, hemisph, horizon, j):
     # populate the sun state (visible or not) for the specified date & latitude
     # note: the first parameter 'dt' is already a datetime 30 seconds before midnight
@@ -879,7 +884,9 @@ def getsunstate(dt, lat, hemisph, horizon, j):
         t0 = ts.ut1(dt.year, dt.month, dt.day, dt.hour, dt.minute, dt.second)
         dt += datetime.timedelta(days=1)
         t9 = ts.ut1(dt.year, dt.month, dt.day, dt.hour, dt.minute, dt.second)
+        start00 = time.time()                   # 00000
         sunstate, y = almanac.find_discrete(t0, t9, daylength(locn, horizon))
+        config.stopwatch2 += time.time()-start00 # 00000
         if len(sunstate) > 0:
 
             if y[0]:
@@ -924,7 +931,7 @@ def getHorizon(t):
     position = earth.at(t).observe(moon)   # at noontime (for daily average distance)
     distance = position.apparent().radec(epoch='date')[2]
     dist_km = distance.km
-    sdm = degrees(atan(1738.1/dist_km))   # equatorial radius of moon = 1738.1 km
+    sdm = math.degrees(math.atan(1738.1/dist_km))   # equatorial radius of moon = 1738.1 km
     horizon = sdm + 0.5666667	# moon's equatorial radius + 34' (atmospheric refraction)
 
     return horizon
@@ -946,13 +953,13 @@ def fetchMoonData(d, tFrom, tNoon, tTo, i, lats, hFlag = False, round2seconds=Fa
         if MoonDate[k] == d:
             ndx = k
             iFnd = True
-        if config.moonDataSeeks >= 31 * MDlen and MoonDate[k] < min_d:
+        if config.moonDaysCount >= 31 * MDlen and MoonDate[k] < min_d:
             min_d = MoonDate[k]     # find oldest date in 'MoonDate'
             idx = k
     if not iFnd:
         # assigining a new index in a rotary fashion works, however a more intelligent algorithm
         #   is to re-use the index to the oldest date once all 'MDlen' dates are in use.
-        if config.moonDataSeeks >= 31 * MDlen:
+        if config.moonDaysCount >= 31 * MDlen:
             # assign a new index by re-using the oldest date
             ndx = idx
             MoonDate[ndx] = d
@@ -972,8 +979,7 @@ def fetchMoonData(d, tFrom, tNoon, tTo, i, lats, hFlag = False, round2seconds=Fa
         horizon = getHorizon(tNoon)
         start00 = time.time()                   # 00000
         moonrise, y = almanac.find_discrete(tFrom, tTo, moonday(locn, horizon))
-        stop00 = time.time()                    # 00000
-        config.stopwatch += stop00-start00      # 00000
+        time00 = time.time()-start00 # 00000
         rise, sett, ris2, set2, fs = rise_set(moonrise,y,lats,round2seconds)
 
         # only store single events (not double events, which are very rare)
@@ -991,6 +997,10 @@ def fetchMoonData(d, tFrom, tNoon, tTo, i, lats, hFlag = False, round2seconds=Fa
         
         #print("np_array[{}][{}][{}] = {}".format(ndx,i-1,0,np_array[ndx][i-1][0]))
         #print("np_array[{}][{}][{}] = {}".format(ndx,i-1,1,np_array[ndx][i-1][1]))
+        if hFlag:
+            config.stopwatch2 += time00
+        else:
+            config.stopwatch += time00
     else:                               # fetch stored data
         rise = np_array[ndx][i-1][0]
         sett = np_array[ndx][i-1][1]
@@ -1056,6 +1066,7 @@ def moonrise_set(d, lat, hemisph):  # used by tables.py in twilighttab (section
     # first compute semi-diameter of moon (in degrees)
     horizon = getHorizon(t0noon)
 
+    config.moonDaysCount += 1
     config.moonDataSeeks += 1
     out[0], out[3], out2[0], out2[3], fs = fetchMoonData(d, t0, t0noon, t1, i, lats)
 
@@ -1063,6 +1074,7 @@ def moonrise_set(d, lat, hemisph):  # used by tables.py in twilighttab (section
         moonvisible[i] = fs
 
     if out[0] == '--:--' and out[3] == '--:--':	# if neither moonrise nor moonset...
+        config.moonDataSeeks -= 1
         if moonvisible[i] == None:
             getmoonstate(dt, lat, hemisph)			# ...get moon state if unknown
         out[0] = moonstate(i)
@@ -1080,6 +1092,7 @@ def moonrise_set(d, lat, hemisph):  # used by tables.py in twilighttab (section
     # first compute semi-diameter of moon (in degrees)
     horizon = getHorizon(t1noon)
 
+    config.moonDaysCount += 1
     config.moonDataSeeks += 1
     out[1], out[4], out2[1], out2[4], fs = fetchMoonData(d1, t1, t1noon, t2, i, lats)
 
@@ -1087,6 +1100,7 @@ def moonrise_set(d, lat, hemisph):  # used by tables.py in twilighttab (section
         moonvisible[i] = fs
 
     if out[1] == '--:--' and out[4] == '--:--':	# if neither moonrise nor moonset...
+        config.moonDataSeeks -= 1
         if moonvisible[i] == None:
             getmoonstate(dt+datetime.timedelta(days=1), lat, hemisph)    # ...get moon state if unknown
         out[1] = moonstate(i)
@@ -1104,6 +1118,7 @@ def moonrise_set(d, lat, hemisph):  # used by tables.py in twilighttab (section
     # first compute semi-diameter of moon (in degrees)
     horizon = getHorizon(t2noon)
 
+    config.moonDaysCount += 1
     config.moonDataSeeks += 1
     out[2], out[5], out2[2], out2[5], fs = fetchMoonData(d2, t2, t2noon, t3, i, lats)
 
@@ -1111,6 +1126,7 @@ def moonrise_set(d, lat, hemisph):  # used by tables.py in twilighttab (section
         moonvisible[i] = fs
 
     if out[2] == '--:--' and out[5] == '--:--':	# if neither moonrise nor moonset...
+        config.moonDataSeeks -= 1
         if moonvisible[i] == None:
             getmoonstate(dt+datetime.timedelta(days=2), lat, hemisph)    # ...get moon state if unknown
         out[2] = moonstate(i)
@@ -1172,7 +1188,9 @@ def getmoonstate(dt, lat, hemisph):
         t0 = ts.ut1(dt.year, dt.month, dt.day, dt.hour, dt.minute, dt.second)
         dt += datetime.timedelta(days=1)
         t9 = ts.ut1(dt.year, dt.month, dt.day, dt.hour, dt.minute, dt.second)
+        start00 = time.time()                   # 00000
         moonrise, y = almanac.find_discrete(t0, t9, moonday(locn, horizon))
+        config.stopwatch2 += time.time()-start00 # 00000
 #        for n in range(len(moonrise)):
 #            print(y[n], moonrise[n].utc_datetime())
         if len(moonrise) > 0:
@@ -1613,9 +1631,9 @@ def moonphase(d):           # used in twilighttab (section 3)
     #nnm = NextNewMoon.replace(tzinfo=None)
 
     if WaxingMoon:
-        phase = pi - phase_angle.radians
+        phase = math.pi - phase_angle.radians
     else:
-        phase = pi + phase_angle.radians
+        phase = math.pi + phase_angle.radians
 
     return phase
 
@@ -1625,7 +1643,7 @@ def moonage(d, d1):         # used in twilighttab (section 3)
     # percent illumination is calculated at noon
     t12 = ts.ut1(d.year, d.month, d.day, 12, 0, 0)
     phase_angle = almanac.phase_angle(eph, 'moon', t12)
-    pctrad = 50 * (1.0 + cos(phase_angle.radians))
+    pctrad = 50 * (1.0 + math.cos(phase_angle.radians))
     pct = "{:.0f}".format(pctrad)
 
     # calculate age of moon
@@ -1694,7 +1712,7 @@ def equation_of_time(d, d1, UpperList, LowerList, extras, round2seconds = False)
         return eqt00,eqt12,mpa12,mp_upper,mp_lower
 
     phase_angle = almanac.phase_angle(eph, 'moon', t12)
-    pctrad = 50 * (1.0 + cos(phase_angle.radians))
+    pctrad = 50 * (1.0 + math.cos(phase_angle.radians))
     pct = "{:.0f}".format(pctrad)
 
     # calculate age of moon
@@ -1795,7 +1813,9 @@ def find_new_moon(d):       # used in doublepage
     d0 = d - datetime.timedelta(days=30)
     t0 = ts.utc(d0.year, d0.month, d0.day, 12, 0, 0)
     t1 = ts.utc(d.year, d.month, d.day, 12, 0, 0)
+    start00 = time.time()                   # 00000
     t, y = almanac.find_discrete(t0, t1, almanac.moon_phases(eph))
+    config.stopwatch += time.time()-start00 # 00000
     for i in range(len(t)):
         if y[i] == 0:       # 0=New Moon, 1=First Quarter, 2=Full Moon, 3=Last Quarter
             PreviousNewMoon = t[i].utc_datetime()
@@ -1808,7 +1828,9 @@ def find_new_moon(d):       # used in doublepage
         # synodic month = about 29.53 days
         t2 = ts.utc(PreviousNewMoon + datetime.timedelta(days=28))
         t3 = ts.utc(PreviousNewMoon + datetime.timedelta(days=30))
+        start00 = time.time()                   # 00000
         t, y = almanac.find_discrete(t2, t3, almanac.moon_phases(eph))
+        config.stopwatch += time.time()-start00 # 00000
         for i in range(len(t)):
             if y[i] == 0:       # 0 = New Moon
                 NextNewMoon = t[i].utc_datetime()
